@@ -18,6 +18,7 @@ import dms_variants._cutils
 from dms_variants.constants import (AAS_NOSTOP,
                                     CODON_TO_AA,
                                     NT_COMPLEMENT,
+                                    SINGLE_NT_AA_MUTS,
                                     )
 
 
@@ -200,7 +201,8 @@ def cumul_rows_by_count(df, *, count_col='count', n_col='n_rows',
 
          # get number of rows with each count
          .assign(**{n_col: 1})
-         .groupby(group_cols + [count_col])
+         .groupby(group_cols + [count_col],
+                  observed=True)
          .aggregate({n_col: 'count'})
          .reset_index()
          .sort_values(group_cols + [count_col],
@@ -211,9 +213,12 @@ def cumul_rows_by_count(df, *, count_col='count', n_col='n_rows',
          .assign(**{n_col: lambda x: x.groupby(group_cols)[n_col].cumsum()})
 
          # add new column that is total number of rows
-         .assign(**{tot_col: lambda x: (x.groupby(group_cols)[n_col]
+         .assign(**{tot_col: lambda x: (x
+                                        .groupby(group_cols, sort=False)
+                                        [n_col]
                                         .transform('max'))})
          )
+    assert df[tot_col].notnull().all()
 
     if drop_group_cols:
         df = df.drop(group_cols, axis='columns')
@@ -691,7 +696,7 @@ def scores_to_prefs(df, mutation_col, score_col, base,
           .assign(unscaled_prefs=lambda x: (base**x[score_col]
                                             )**stringency_param,
                   preference=lambda x: (x['unscaled_prefs'] /
-                                        (x.groupby('site')
+                                        (x.groupby('site', sort=False)
                                          ['unscaled_prefs']
                                          .transform('sum')
                                          )
@@ -720,6 +725,55 @@ def scores_to_prefs(df, mutation_col, score_col, base,
     return df
 
 
+def single_nt_accessible(codon, aa, codon_encode_aa='raise'):
+    """Is amino acid accessible from codon by single-nucleotide change?
+
+    Parameters
+    ----------
+    codon : str
+        The codon.
+    aa : str
+        The amino acid.
+    codon_encode_aa : {'raise', 'true', 'false'}
+        If `codon` encodes `aa`, raise an error, return `True`,
+        or return `False`.
+
+    Returns
+    -------
+    bool
+
+    Example
+    -------
+    >>> single_nt_accessible('GGG', 'E')
+    True
+    >>> single_nt_accessible('GGC', 'E')
+    False
+    >>> single_nt_accessible('GGG', 'G')
+    Traceback (most recent call last):
+      ...
+    ValueError: `codon` GGG already encodes `aa` G (see `codon_encode_aa`)
+    >>> single_nt_accessible('GGG', 'G', codon_encode_aa='true')
+    True
+    >>> single_nt_accessible('TTT', 'L')
+    True
+
+    """
+    if CODON_TO_AA[codon] == aa:
+        if codon_encode_aa == 'raise':
+            raise ValueError(f"`codon` {codon} already encodes `aa` {aa} "
+                             '(see `codon_encode_aa`)')
+        elif codon_encode_aa == 'false':
+            return False
+        elif codon_encode_aa == 'true':
+            return True
+        else:
+            raise ValueError(f"invalid `codon_encode_aa` {codon_encode_aa}")
+    elif aa in SINGLE_NT_AA_MUTS[codon]:
+        return True
+    else:
+        return False
+
+
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
